-- scope stuff
qualityoflife = qualityoflife or {};
local q = qualityoflife;
q.modules = q.modules or {};
q.name = "QoL_Core";
q.version = 6; -- check for toc

-- variables
local partyMembers, raidMembers;

-- init
function q:Init()
	if not q.initFinished then
		q:InitOptions();

		q.initFinished = true;
		q:Println(q:GetText("AddOn loaded. Type /qol for help."));

		q:LoadModules();
	end
end

function q:LoadModules()
	for id = 1, GetNumAddOns() do
		local name, title, notes, loadable, reason, security, newVersion = GetAddOnInfo(id);
		if q:StartsWith(name, "QoL") and name ~= q.name then
			if reason == "DEMAND_LOADED" then
				local loaded, reason = LoadAddOn(id);
				if not loaded then
					q:Println(q:GetText("Failed to load module \"%s\": %s", name, reason));
				end
			elseif reason ~= "DISABLED" then
				q:Println(q:GetText("Failed to load module \"%s\": %s", name, reason));
			end
		end
	end
end

-- command handling
SLASH_QOL1 = "/qol";
local CMD_RESET = "reset";

SlashCmdList["QOL"] = function(msg)
	if msg and msg == CMD_RESET then
		q:Reset();
		return;
	end

	q:Println(q:GetText("Unknown command. Valid commands are:"));
	q:Println(q:GetText("/qol reset (resets the addon to defaults)"));
end

-- event handling
local eventFrame;
function q:GetEventFrame()
	eventFrame = eventFrame or CreateFrame("Frame", "QolEventFrame");
	return eventFrame;
end

q:GetEventFrame():RegisterEvent("ADDON_LOADED");
q:GetEventFrame():RegisterEvent("GROUP_ROSTER_UPDATE");
q:GetEventFrame():RegisterEvent("RAID_ROSTER_UPDATE");

q:GetEventFrame():SetScript("OnEvent", function(self, event, arg1)
	if event then
		if event == "ADDON_LOADED" and arg1 == "QoL_Core" and not q.initFinished then
			q:Init();
		elseif event == "GROUP_ROSTER_UPDATE" or event == "RAID_ROSTER_UPDATE" then
			q:ResetMemberInfo();
		elseif event == "PLAYER_ENTERING_WORLD" then
			C_Timer.After(1, function () q:SetCrfVisible(QOL_OPTIONS[q.OPT_SHOWCRF]); end);
		end
	end
end);

-- core stuff
function q:SetCrfVisible(value)
	if _G["CompactRaidFrameContainer"] and _G["CompactRaidFrameContainer"]:IsVisible() ~= value then
		if value then
			_G["CompactRaidFrameContainer"]:Show();
		else
			_G["CompactRaidFrameContainer"]:Hide();
		end
	end
end

function q:GetCharName()
	return GetUnitName("player");
end

function q:GetPartyMembers()
	if not partyMembers then
		partyMembers = {};
		local plist = GetHomePartyInfo();

		if plist then
			local n = 1;
			for _, v in pairs(plist) do
				partyMembers[n] = v;
				n = n + 1;
			end
		end
	end
	return partyMembers;
end

function q:GetRaidMembers()
	if not raidMembers then
		raidMembers = { [0] = {} };
		local noRaidMembers = GetNumGroupMembers();

		if noRaidMembers then
			for n = 1, noRaidMembers do
				local name, rank, subgroup, level, class, fileName, zone, online, isDead, role, isML = GetRaidRosterInfo(n);

				-- table of all raid members by id
				raidMembers[0][n] = name;

				-- table of all raid members, mapped by subgroups
				if not raidMembers[subgroup] then
					raidMembers[subgroup] = {};
				end
				local subGroupId = table.getn(raidMembers[subgroup]) + 1;
				raidMembers[subgroup][subGroupId] = name;

				n = n + 1;
			end
		end
	end
	return raidMembers;
end

function q:ResetMemberInfo()
	partyMembers = nil;
	raidMembers = nil;
end

function q:Reset()
	local success = true;

	QOL_OPTIONS = {};
	if q.InitOptions then
		q:InitOptions();
	else
		success = false;
	end
	q:Println(q:GetText("%s has been resetted to defaults.", q.name));

	for name, addon in pairs(q.modules) do
		q:Println(q:GetText("%s has been resetted to defaults.", name));
		if addon.InitOptions then
			addon:InitOptions();
		else
			success = false;
		end
	end

	if not success then
		q:Println(q:GetText("You might need to relog in order to finish the process."));
	end
end